#region Copyright (C) Clever Components
//Clever Internet Suite
//Copyright (C) 2013 Clever Components
//All Rights Reserved
//www.CleverComponents.com
#endregion

using System;
using System.IO;
using System.ComponentModel;
using System.Drawing;

namespace CleverComponents.InetSuite
{

	[ToolboxBitmap(typeof(SmtpFileHandler), "Resources.SmtpFileHandler.bmp")]
	public class SmtpFileHandler : Component
	{
		public static int MaxTryCount = 1000;
		public static string MessageFileExt = ".MSG";
		public static string EnvelopeFileExt = ".ENV";

		SmtpServer server;
		string mailBoxDir;
		string relayDir;
		int counter;

		void DoMessageRelayed(object sender, SmtpMessageReceivedEventArgs e)
		{
			try
			{
				string path = FileUtils.AddTrailingBackSlash(RelayDir);

				lock (this)
				{
					Directory.CreateDirectory(path);

					string msgFileName = String.Empty;
					int i = 0;
					while (true)
					{
						counter++;
						msgFileName = path + String.Format("MAIL{0,8}", counter).Replace(' ', '0');

						try
						{
							if (!File.Exists(msgFileName + MessageFileExt))
							{
								using (Stream stream = new FileStream(msgFileName + MessageFileExt, FileMode.CreateNew, FileAccess.Write))
								{
									e.Message.SaveToStream(stream, String.Empty);
									break;
								}
							}
						}
						catch (IOException) { }
						i++;
						if (i > MaxTryCount)
						{
							throw new Exception("Cannot save new message");
						}
					}

					System.Diagnostics.Debug.Assert(!StringUtils.IsEmpty(msgFileName));

					StringCollectionEx envelope = new StringCollectionEx();
					envelope.Add(e.MailFrom);
					envelope.AddRange(e.Recipients);

					using (Stream stream = new FileStream(msgFileName + EnvelopeFileExt, FileMode.CreateNew, FileAccess.Write))
					{
						envelope.SaveToStream(stream, String.Empty);
					}
				}

				e.Action = SmtpMailDataAction.Ok;
			}
			catch
			{
				e.Action = SmtpMailDataAction.ProcessingError;
			}
		}

		void DoMessageDelivered(object sender, SmtpMessageDeliveredEventArgs e)
		{
			try
			{
				System.Diagnostics.Debug.Assert(e.Account != null);

				string path = GetMailBoxPath(e.Account.UserName);

				lock (this)
				{
					Directory.CreateDirectory(path);

					int i = 0;
					while (true)
					{
						counter++;
						string msgFileName = path + String.Format("MAIL{0,8}", counter).Replace(' ', '0');

						try
						{
							if (!File.Exists(msgFileName + MessageFileExt))
							{
								using (Stream stream = new FileStream(msgFileName + MessageFileExt, FileMode.CreateNew, FileAccess.Write))
								{
									e.Message.SaveToStream(stream, String.Empty);
									break;
								}
							}
						}
						catch (IOException) { }
						i++;
						if (i > MaxTryCount)
						{
							throw new Exception("Cannot save new message");
						}
					}
				}

				e.Action = SmtpMailDataAction.Ok;
			}
			catch
			{
				e.Action = SmtpMailDataAction.ProcessingError;
			}
		}

		string GetMailBoxPath(string userName)
		{
			return FileUtils.AddTrailingBackSlash(MailBoxDir) + FileUtils.AddTrailingBackSlash(userName);
		}


		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				CleanEventHandlers();
			}
			base.Dispose(disposing);
		}

		protected virtual void CleanEventHandlers()
		{
			if (server != null)
			{
				server.MessageRelayed -= new SmtpMessageReceivedEventHandler(DoMessageRelayed);
				server.MessageDelivered -= new SmtpMessageDeliveredEventHandler(DoMessageDelivered);
			}
		}

		protected virtual void InitEventHandlers()
		{
			if (server != null)
			{
				server.MessageRelayed += new SmtpMessageReceivedEventHandler(DoMessageRelayed);
				server.MessageDelivered += new SmtpMessageDeliveredEventHandler(DoMessageDelivered);
			}
		}

		public SmtpFileHandler() : base()
		{
			counter = 1;
			mailBoxDir = "";
			relayDir = "";
		}

		[DefaultValue(null)]
		public SmtpServer Server
		{
			get { return server; }
			set
			{
				CleanEventHandlers();
				server = value;
				InitEventHandlers();
			}
		}

		[DefaultValue("")]
		public string MailBoxDir
		{
			get { return mailBoxDir; }
			set
			{
				lock (this)
				{
					mailBoxDir = value;
				}
			}
		}

		[DefaultValue("")]
		public string RelayDir
		{
			get { return relayDir; }
			set
			{
				lock (this)
				{
					relayDir = value;
				}
			}
		}

		[DefaultValue(1)]
		public int Counter
		{
			get { return counter; }
			set
			{
				lock (this)
				{
					counter = value;
				}
			}
		}
	}
}
